Package: cageminer
Title: Candidate Gene Miner
Version: 1.18.0
Authors@R: 
    c(
    person(given = "Fabrício",
           family = "Almeida-Silva",
           role = c("aut", "cre"),
           email = "fabricio_almeidasilva@hotmail.com",
           comment = c(ORCID = "0000-0002-5314-2964")),
    person(given = "Thiago",
           family = "Venancio",
           role = "aut",
           comment = c(ORCID = "0000-0002-2215-8082"))
    )
Description: This package aims to integrate GWAS-derived SNPs and coexpression
    networks to mine candidate genes associated with a particular phenotype.
    For that, users must define a set of guide genes, which are known genes 
    involved in the studied phenotype. Additionally, the mined candidates can
    be given a score that favor candidates that are hubs and/or transcription
    factors. The scores can then be used to rank and select the top n most
    promising genes for downstream experiments.
License: GPL-3
URL: https://github.com/almeidasilvaf/cageminer
BugReports: https://support.bioconductor.org/t/cageminer
biocViews: Software, SNP, FunctionalPrediction, GenomeWideAssociation,
        GeneExpression, NetworkEnrichment, VariantAnnotation,
        FunctionalGenomics, Network
Encoding: UTF-8
LazyData: false
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: ggplot2, rlang, ggbio, ggtext, GenomeInfoDb, GenomicRanges,
        IRanges, reshape2, methods, BioNERO
Depends: R (>= 4.1)
Suggests: testthat (>= 3.0.0), SummarizedExperiment, knitr, BiocStyle,
        rmarkdown, covr, sessioninfo
Config/testthat/edition: 3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/cageminer
git_branch: RELEASE_3_23
git_last_commit: 2f4c34c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:02:55 UTC; biocbuild
Author: Fabrício Almeida-Silva [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5314-2964>),
  Thiago Venancio [aut] (ORCID: <https://orcid.org/0000-0002-2215-8082>)
Maintainer: Fabrício Almeida-Silva <fabricio_almeidasilva@hotmail.com>

# Generated by roxygen2: do not edit by hand

export(mine_candidates)
export(mine_step1)
export(mine_step2)
export(mine_step3)
export(plot_snp_circos)
export(plot_snp_distribution)
export(score_genes)
export(simulate_windows)
importFrom(BioNERO,gene_significance)
importFrom(BioNERO,module_enrichment)
importFrom(GenomeInfoDb,keepSeqlevels)
importFrom(GenomeInfoDb,seqinfo)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqlevels)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,countOverlaps)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,mcols)
importFrom(GenomicRanges,tileGenome)
importFrom(GenomicRanges,width)
importFrom(IRanges,subsetByOverlaps)
importFrom(ggbio,circle)
importFrom(ggbio,ggbio)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggtext,element_textbox_simple)
importFrom(methods,is)
importFrom(reshape2,melt)
importFrom(rlang,.data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cageminer <img src='man/figures/logo.png' align="right" height="139" />

<!-- badges: start -->

[![GitHub
issues](https://img.shields.io/github/issues/almeidasilvaf/cageminer)](https://github.com/almeidasilvaf/cageminer/issues)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check-bioc](https://github.com/almeidasilvaf/cageminer/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/almeidasilvaf/cageminer/actions)
[![Codecov test
coverage](https://codecov.io/gh/almeidasilvaf/cageminer/branch/devel/graph/badge.svg)](https://codecov.io/gh/almeidasilvaf/cageminer?branch=devel)
<!-- badges: end -->

The goal of `cageminer` is to integrate SNP data from GWAS results with
gene coexpression networks to identify high-confidence candidate genes
involved in a particular phenotype. To identify high-confidence
candidate genes, `cageminer` considers 3 criteria:

1.  Physical proximity (or linkage disequilibrium with) trait-related
    SNPs;
2.  Presence in coexpression modules enriched in guide genes (i.e.,
    “reference” genes that are known to be associated with the
    phenotype).
3.  Significant altered expression levels in a condition of interest
    (e.g., stress, disease, etc).

By default, `cageminer` defines genes as high-confidence candidates if
they satisfy all of the 3 criteria above, but users can choose to use
only one/some of them.

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install `cageminer` from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("cageminer")
```

And the development version from
[GitHub](https://github.com/almeidasilvaf/cageminer) with:

``` r
BiocManager::install("almeidasilvaf/cageminer")
```

## Citation

Below is the citation output from using `citation('cageminer')` in R.
Please run this yourself to check for any updates on how to cite
**cageminer**.

``` r
print(citation('cageminer'), bibtex = TRUE)
#> 
#> To cite cageminer in publications use:
#> 
#>   Almeida-Silva, F., & Venancio, T. M. (2022). cageminer: an
#>   R/Bioconductor package to prioritize candidate genes by integrating
#>   genome-wide association studies and gene coexpression networks. in
#>   silico Plants, 4(2), diac018.
#>   https://doi.org/10.1093/insilicoplants/diac018
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {cageminer: an R/Bioconductor package to prioritize candidate genes by integrating genome-wide association studies and gene coexpression networks},
#>     author = {Fabricio Almeida-Silva and Thiago M. Venancio},
#>     journal = {in silico Plants},
#>     year = {2022},
#>     volume = {4},
#>     number = {2},
#>     pages = {diac018},
#>     url = {https://doi.org/10.1093/insilicoplants/diac018},
#>     doi = {10.1093/insilicoplants/diac018},
#>   }
```

## Code of Conduct

Please note that the `cageminer` project is released with a [Contributor
Code of Conduct](http://bioconductor.org/about/code-of-conduct/). By
contributing to this project, you agree to abide by its terms.

## Development tools

- Continuous code testing is possible thanks to [GitHub
  actions](https://www.tidyverse.org/blog/2020/04/usethis-1-6-0/)
  through *[usethis](https://CRAN.R-project.org/package=usethis)*,
  *[remotes](https://CRAN.R-project.org/package=remotes)*, and
  *[rcmdcheck](https://CRAN.R-project.org/package=rcmdcheck)* customized
  to use [Bioconductor’s docker
  containers](https://www.bioconductor.org/help/docker/) and
  *[BiocCheck](https://bioconductor.org/packages/3.15/BiocCheck)*.
- Code coverage assessment is possible thanks to
  [codecov](https://codecov.io/gh) and
  *[covr](https://CRAN.R-project.org/package=covr)*.
- The [documentation website](http://almeidasilvaf.github.io/cageminer)
  is automatically updated thanks to
  *[pkgdown](https://CRAN.R-project.org/package=pkgdown)*.
- The code is styled automatically thanks to
  *[styler](https://CRAN.R-project.org/package=styler)*.
- The documentation is formatted thanks to
  *[devtools](https://CRAN.R-project.org/package=devtools)* and
  *[roxygen2](https://CRAN.R-project.org/package=roxygen2)*.

For more details, check the `dev` directory.

This package was developed using
*[biocthis](https://bioconductor.org/packages/3.15/biocthis)*.

#' Simulate number of genes for each sliding window
#'
#' This function counts genes that are contained in sliding windows related to
#' each SNP.
#'
#' @param gene_ranges A GRanges object with genomic coordinates
#' of all genes in the genome.
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#' @param windows Sliding windows (in Mb) upstream and downstream relative
#' to each SNP. Default: seq(0.1, 2, by = 0.1).
#' @param expand_intervals Logical indicating whether or not to expand markers
#' that are represented by intervals. This is particularly useful
#' if users want to use a custom interval defined by linkage disequilibrium,
#' for example. Default: TRUE.
#'
#' @return A ggplot object summarizing the results of the simulations.
#' @details
#' By default, the function creates 20 sliding windows by expanding upstream
#' and downstream boundaries for each SNP from 0.1 Mb (100 kb) to 2 Mb.
#' @seealso
#'  \code{\link[IRanges]{findOverlaps-methods}}
#' @rdname simulate_windows
#' @export
#' @importFrom IRanges subsetByOverlaps
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot aes geom_line geom_point scale_color_manual labs theme element_text
#' @importFrom methods is
#' @examples
#' data(snp_pos)
#' data(gene_ranges)
#' simulate_windows(gene_ranges, snp_pos)
simulate_windows <- function(gene_ranges, marker_ranges,
                             windows = seq(0.1, 2, by=0.1),
                             expand_intervals = TRUE) {
    if(is(marker_ranges, "GRanges")) {
        sim <- lapply(windows, function(y) {
            return(handle_intervals(marker_ranges, y,
                                    expand_intervals = TRUE))
        })
        gene_count <- data.frame(count=unlist(lapply(sim, function(x) {
            return(length(unique(IRanges::subsetByOverlaps(gene_ranges, x))))
        })))
        legend_pos <- "none"
    } else {
        sim <- lapply(marker_ranges, function(x) {
            runs <- lapply(windows, function(y) {
                return(handle_intervals(x, y,
                                        expand_intervals = TRUE))
            })
            return(runs)
        })
        gene_count <- as.data.frame(lapply(sim, function(x) {
            count <- unlist(lapply(x, function(y) {
                return(length(unique(IRanges::subsetByOverlaps(gene_ranges, y))))
            }))
            return(count)
        }))
        legend_pos <- "bottom"
    }
    gene_count$windows <- as.factor(seq(0.1, 2, by = 0.1))
    gene_count_melt <- reshape2::melt(gene_count, id.vars = "windows")

    p <- ggplot(
        gene_count_melt, aes(
            x = .data$windows, y = .data$value,
            group = .data$variable, color = .data$variable
        )
    ) +
        ggplot2::geom_line() +
        ggplot2::geom_point() +
        ggplot2::scale_color_manual(values = custom_pal(1)) +
        ggplot2::labs(x="Sliding windows (Mb)", y="Gene count",
                      title="Genes per sliding window",
                      color="Trait") +
        ggplot_theme() +
        ggplot2::theme(legend.position = legend_pos)
    return(p)
}

#----Load data------------------------------------------------------------------
data(snp_pos)
data(gene_ranges)

#----Start tests----------------------------------------------------------------
test_that("simulate_windows() plots number of genes per sliding window", {
    snp_pos_list <- GenomicRanges::GRangesList(
        snp_pos[1:20],
        snp_pos[21:40],
        snp_pos[41:60]
    )
    p <- simulate_windows(gene_ranges, snp_pos)
    p2 <- simulate_windows(gene_ranges, snp_pos_list)

    expect_true("ggplot" %in% class(p))
    expect_true("ggplot" %in% class(p2))
})

#' Set custom ggplot2 theme
#'
#' @return Theme for ggplot graphics.
#' @noRd
#' @importFrom ggplot2 theme_bw theme element_text element_blank
ggplot_theme <- function() {
    theme <- ggplot2::theme_bw() +
        ggplot2::theme(
        plot.title = ggplot2::element_text(face="bold", size=12),
        panel.grid.major = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank()
    )
    return(theme)
}


#' Custom color palette
#'
#' @param pal Color palette to use. One of 1 or 2. Default: 1.
#' @return A color palette as a character vector.
#' @noRd
custom_pal <- function(pal=1) {
    if(pal == 1) {
        col <- c("#E64B35FF", "#4DBBD5FF", "#00A087FF", "#3C5488FF",
                 "#F39B7FFF", "#8491B4FF", "#91D1C2FF", "#DC0000FF",
                 "#7E6148FF", "#B09C85FF")
    } else if(pal == 2) {
        col <- c("#393B79FF", "#637939FF", "#8C6D31FF", "#843C39FF",
                 "#7B4173FF", "#5254A3FF", "#8CA252FF", "#BD9E39FF",
                 "#AD494AFF", "#A55194FF", "#6B6ECFFF", "#B5CF6BFF",
                 "#E7BA52FF", "#D6616BFF", "#CE6DBDFF", "#9C9EDEFF",
                 "#CEDB9CFF", "#E7CB94FF", "#E7969CFF", "#DE9ED6FF")
    } else {
        stop("pal must be 1 or 2.")
    }
    return(col)
}



#' Wrapper function to check input for plot_snp_circos()
#'
#' @param genome_ranges A GRanges object with chromosome lengths.
#' @param gene_ranges A GRanges object with genomic coordinates
#' of all genes in the genome.
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#'
#' @return If input objects are not as expected, it will throw an error.
#' Otherwise, nothing happens.
#' @noRd
check_input_circos <- function(genome_ranges, gene_ranges, marker_ranges) {
    if(!methods::is(genome_ranges, "GRanges")) {
        stop("Argument 'genome_ranges' must be a GRanges object.")
    }
    if(!methods::is(gene_ranges, "GRanges")) {
        stop("Argument 'gene_ranges' must be a GRanges object.")
    }
    if(!class(marker_ranges) %in%
       c("GRanges", "GRangesList", "CompressedGRangesList")) {
        stop(
            "Argument 'marker_ranges' must be a GRanges, GRangesList or
            CompressedGRangesList object."
            )
    }
}


#' Wrapper function to add seqlengths to ranges for plot_snp_circos()
#'
#' @param genome_ranges A GRanges object with chromosome lengths.
#' @param gene_ranges A GRanges object with genomic coordinates
#' of all genes in the genome.
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#'
#' @return The genome_ranges, gene_ranges, or marker_ranges object with
#' seqlengths included.
#' @importFrom GenomeInfoDb seqlevels seqlengths keepSeqlevels
#' @importFrom GenomicRanges end
#' @noRd
add_seqlen <- function(genome_ranges, gene_ranges, marker_ranges) {
    GenomeInfoDb::seqlengths(gene_ranges) <- GenomicRanges::end(genome_ranges)
    GenomeInfoDb::seqlengths(genome_ranges) <- GenomicRanges::end(genome_ranges)

    levels <- GenomeInfoDb::seqlevels(marker_ranges)
    filt_genome <- GenomeInfoDb::keepSeqlevels(genome_ranges, levels,
                                               pruning.mode = "tidy")
    GenomeInfoDb::seqlengths(marker_ranges) <- GenomicRanges::end(filt_genome)
    result <- list(genome = genome_ranges,
                   genes = gene_ranges,
                   markers = marker_ranges)
    return(result)
}

#' Circos plot of SNP distribution across chromosomes
#'
#' @param genome_ranges A GRanges object with chromosome lengths.
#' @param gene_ranges A GRanges object with genomic coordinates
#' of all genes in the genome.
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#'
#' @return A ggplot object with a circos plot of molecular marker distribution
#' across chromosomes.
#' @rdname plot_snp_circos
#' @export
#' @importFrom rlang .data
#' @importFrom GenomeInfoDb seqinfo
#' @importFrom ggplot2 labs theme margin aes scale_fill_gradient
#' @importFrom ggtext element_textbox_simple
#' @importFrom GenomicRanges end tileGenome countOverlaps
#' @importFrom ggbio circle ggbio
#' @examples
#' data(snp_pos)
#' data(gene_ranges)
#' data(chr_length)
#' p <- plot_snp_circos(chr_length, gene_ranges, snp_pos)
plot_snp_circos <- function(genome_ranges, gene_ranges, marker_ranges) {
    check_input_circos(genome_ranges, gene_ranges, marker_ranges)
    pal <- c("#1F77B4FF", "#FF7F0EFF", "#2CA02CFF", "#D62728FF", "#9467BDFF",
             "#8C564BFF", "#E377C2FF", "#7F7F7FFF", "#BCBD22FF", "#17BECFFF")
    gene_ranges <- add_seqlen(genome_ranges, gene_ranges, marker_ranges)$genes
    genome_ranges <- add_seqlen(genome_ranges,gene_ranges,marker_ranges)$genome
    marker_ranges <- add_seqlen(genome_ranges,gene_ranges,marker_ranges)$markers
    windows <- GenomicRanges::tileGenome(GenomeInfoDb::seqinfo(gene_ranges),
                                         tilewidth = 10^6,
                                         cut.last.tile.in.chrom = TRUE)
    windows$ngenes <- GenomicRanges::countOverlaps(windows, gene_ranges)
    p <- ggbio::ggbio()
    if(!is(marker_ranges, "GRanges")) {
        traits <- paste0(unlist(lapply(seq_along(marker_ranges), function(i) {
            return(paste0(
                "<span style='color:", pal[i], "'>",
                names(marker_ranges)[i],"</span>"))
        })), collapse=", ")
        for(n in seq_along(marker_ranges)) {
            p <- p + ggbio::circle(marker_ranges[[n]], geom = "rect", color = pal[n])
        }
    } else {

        traits <- "trait"
        p <- p + ggbio::circle(marker_ranges, geom = "rect", color = pal[1])
    }
    title <- paste0(
        "<b>SNP distribution across chromosomes</b><br>",
        "<span style = 'font-size:10pt'>Gene density and SNPs associated with ",
        traits, ".</span>")
    p <- p +
        ggbio::circle(
            windows, geom = "bar", stat="identity",
            aes(fill = .data$ngenes, y = .data$ngenes),
            color = NA
        ) +
        ggplot2::scale_fill_gradient(low="#74c476", high="darkgreen") +
        ggbio::circle(genome_ranges, geom = "scale", size = 2) +
        ggbio::circle(
            genome_ranges, geom = "text",
            aes(label = .data$seqnames), vjust = -1, size = 3
        ) +
        labs(title = title, fill = "Gene frequency") +
        theme(
            plot.title.position = "plot",
            plot.title = ggtext::element_textbox_simple(
                size = 13,
                lineheight = 1,
                padding = ggplot2::margin(5.5, 5.5, 5.5, 5.5),
                margin = ggplot2::margin(0, 0, 5.5, 0),
                )
        )

    return(p)
}

#' Plot a barplot of SNP distribution across chromosomes
#'
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait. List elements must have names for proper labelling.
#' @return A ggplot object.
#' @rdname plot_snp_distribution
#' @export
#' @importFrom ggplot2 ggplot aes geom_bar facet_wrap coord_flip theme_bw
#' ggtitle theme element_text element_blank scale_fill_manual
#' @importFrom methods is
#' @examples
#' data(snp_pos)
#' p <- plot_snp_distribution(snp_pos)
plot_snp_distribution <- function(marker_ranges) {
    if(!class(marker_ranges) %in% c("GRanges","GRangesList",
                                    "CompressedGRangesList")) {
        stop("Argument 'marker_ranges' must be a GRanges,
             GRangesList, or CompressedGRangesList object.")
    }
    if(methods::is(marker_ranges, "GRanges")) {
        marker_df <- as.data.frame(marker_ranges)
        marker_df <- as.data.frame(table(marker_df$seqnames))
        colnames(marker_df) <- c("Chromosome", "Frequency")
        wrap <- NULL
        bar <- ggplot2::geom_bar(stat = "identity")
        cols <- NULL
    } else {
        marker_df <- lapply(marker_ranges, as.data.frame)
        marker_df <- Reduce(rbind, lapply(seq_along(marker_df), function(x) {
            return(cbind(marker_df[[x]], names(marker_df)[x]))
        }))
        marker_df <- as.data.frame(table(marker_df[[ncol(marker_df)]],
                                         marker_df[["seqnames"]]))
        colnames(marker_df) <- c("Trait", "Chromosome", "Frequency")
        ntr <- nlevels(marker_df$Trait)
        wrap <- facet_wrap(~Trait, ncol = ntr)
        bar <- geom_bar(aes(fill = .data$Trait), stat = "identity",
                                 show.legend = FALSE)
        cols <- ggplot2::scale_fill_manual(values = custom_pal(2)[seq_len(ntr)])
    }
    p <- ggplot(marker_df, aes(x = .data$Chromosome, y = .data$Frequency)) +
        bar + cols + wrap +
        coord_flip() +
        ggplot2::ggtitle("SNP distribution across chromosomes") +
        ggplot_theme()
    return(p)
}

#----Load data------------------------------------------------------------------
data(snp_pos)
data(gene_ranges)
data(chr_length)

## Simulate GRangesList of multiple traits
snp_pos_list <- GenomicRanges::GRangesList(
    snp_pos[1:20],
    snp_pos[21:40],
    snp_pos[41:60],
    snp_pos[61:80],
    snp_pos[81:100],
    snp_pos[101:116]
)
names(snp_pos_list) <- c("Trait1", "Trait2", "Trait3", "Trait4",
                         "Trait5", "Trait6")


#----Start tests----------------------------------------------------------------
test_that("custom_pal() returns a color palette in a character vector", {
    pal1 <- custom_pal(1)
    pal2 <- custom_pal(2)

    expect_equal(class(pal1), "character")
    expect_equal(class(pal2), "character")
    expect_error(custom_pal(3))
})

test_that("check_input_circos() can properly check input data classes", {

    expect_error(
        check_input_circos(as.data.frame(chr_length))
    )

    expect_error(
        check_input_circos(chr_length, gene_ranges = as.data.frame(gene_ranges))
    )

    expect_error(
        check_input_circos(
            chr_length, gene_ranges, as.data.frame(snp_pos)
        )
    )
})

test_that("plot_snp_circos() plots SNP positions in the genome", {
    p1 <- plot_snp_circos(chr_length, gene_ranges, snp_pos)
    p2 <- plot_snp_circos(chr_length, gene_ranges, snp_pos_list)
    expect_true(length(p1) >0)
    expect_true(length(p2) >0)
})

test_that("plot_snp_distribution() plots SNP distribution", {
    p <- plot_snp_distribution(snp_pos)
    p2 <- plot_snp_distribution(snp_pos_list)

    expect_true("ggplot" %in% class(p))
    expect_true("ggplot" %in% class(p2))
    expect_error(plot_snp_distribution(as.data.frame(snp_pos)))
})

#' Capsicum annuum SNPs associated with resistance to Phytophthora root rot.
#'
#' The SNPs in this data set were retrieved from Siddique et al., 2019, and
#' they are associated to resistance to Phytophthora root rot.
#'
#' @name snp_pos
#' @format A GRanges object.
#' @references
#' Siddique, M.I., Lee, HY., Ro, NY. et al. Identifying candidate genes for
#' Phytophthora capsici resistance in pepper (Capsicum annuum) via
#' genotyping-by-sequencing-based QTL mapping and genome-wide association
#' study. Sci Rep 9, 9962 (2019). https://doi.org/10.1038/s41598-019-46342-1
#' @examples
#' data(snp_pos)
#' @usage data(snp_pos)
"snp_pos"


#' Genomic coordinates of pepper genes
#'
#' GRanges object with genomic coordinates of pepper genes downloaded from
#' http://peppergenome.snu.ac.kr/download.php.
#'
#' @name gene_ranges
#' @format A GRanges object
#' @examples
#' data(gene_ranges)
#' @usage data(gene_ranges)
"gene_ranges"


#' Pepper chromosome lengths
#'
#' Lengths of pepper chromosomes 1-12 in a GRanges object.
#' The genome for which lengths were calculated (v1.55) was downloaded from
#' http://peppergenome.snu.ac.kr/download.php
#'
#' @name chr_length
#' @format A GRanges object
#' @examples
#' data(chr_length)
#' @usage data(chr_length)
"chr_length"


#' Gene expression data from Kim et al., 2018.
#'
#' The data were filtered to keep only the top 4000 genes with highest RPKM
#' values in PRR stress-related samples.
#'
#' @name pepper_se
#' @format A SummarizedExperiment object.
#' @references
#' Kim, MS., Kim, S., Jeon, J. et al. Global gene expression profiling for
#' fruit organs and pathogen infections in the pepper, Capsicum annuum L..
#' Sci Data 5, 180103 (2018). https://doi.org/10.1038/sdata.2018.103
#' @examples
#' data(pepper_se)
#' @usage data(pepper_se)
"pepper_se"


#' Guide genes associated with defense and resistance to oomycetes
#'
#' The GO annotation was retrieved from PLAZA 4.0 Dicots.
#'
#' @name guides
#' @format A data frame with genes in the first column and GO description
#' in the second column.
#' @references
#' Van Bel, M., Diels, T., Vancaester, E., Kreft, L., Botzki, A.,
#' Van de Peer, Y., ... & Vandepoele, K. (2018). PLAZA 4.0: an integrative
#' resource for functional, evolutionary and comparative plant
#' genomics. Nucleic acids research, 46(D1), D1190-D1196.
#' @examples
#' data(guides)
#' @usage data(guides)
"guides"

#' Pepper transcription factors
#'
#' Pepper transcription factors and their families retrieved from PlantTFDB 4.0.
#'
#' @name tfs
#' @format A data frame with gene IDs in the first column and TF families in
#' the second column.
#' @references
#' Jin, J., Tian, F., Yang, D. C., Meng, Y. Q., Kong, L., Luo, J., &
#' Gao, G. (2016). PlantTFDB 4.0: toward a central hub for transcription
#' factors and regulatory interactions in plants. Nucleic acids research, gkw982.
#' @examples
#' data(tfs)
#' @usage data(tfs)
"tfs"


#' Simulation of the output list from BioNERO::exp2gcn() with pepper data
#'
#' This object is a list as returned by BioNERO::exp2gcn(), but only the
#' element genes_and_modules is included. For running time issues,
#' only genes in the cyan module were kept in the element genes_and_modules.
#' All other list elements have been assigned NULL.
#' The network was inferred using the code from the vignette.
#'
#' @name gcn
#' @format A list with the elements returned by BioNERO::exp2gcn().
#' @examples
#' data(gcn)
#' @usage data(gcn)
"gcn"


#' Example output from mined_candidates()
#'
#' The data frame was created using the code from the vignette.
#'
#' @name mined_candidates
#' @format Data frame with an example of the output from mined_candidates
#' @examples
#' data(mined_candidates)
#' @usage data(mined_candidates)
"mined_candidates"

#' Example output from mine_step2()
#'
#' The list was created using the example code from mine_step().
#'
#' @name mine2
#' @format List with elements 'candidates' (character vector) and 'enrichment'
#' (data frame).
#' @examples
#' data(mine2)
#' @usage data(mine2)
"mine2"

#' Example hub genes for the network stored in the gcn object
#'
#' The data frame was created using the code from the vignette.
#'
#' @name hubs
#' @format Data frame with gene IDs, module and intramodular degree.
#' @examples
#' data(hubs)
#' @usage data(hubs)
"hubs"

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL
)

## ----cit, eval = requireNamespace('cageminer')--------------------------------
print(citation('cageminer'), bibtex = TRUE)

## ----installation, eval=FALSE-------------------------------------------------
# if(!requireNamespace('BiocManager', quietly = TRUE))
#   install.packages('BiocManager')
# BiocManager::install("cageminer")

## ----load_package, message=FALSE----------------------------------------------
# Load package after installation
library(cageminer)
set.seed(123) # for reproducibility

## ----data_description---------------------------------------------------------
# GRanges of SNP positions
data(snp_pos)
snp_pos

# GRanges of chromosome lengths
data(chr_length) 
chr_length

# GRanges of gene coordinates
data(gene_ranges) 
gene_ranges

# SummarizedExperiment of pepper response to Phytophthora root rot (RNA-seq)
data(pepper_se)
pepper_se

## ----snp_dist-----------------------------------------------------------------
plot_snp_distribution(snp_pos)

## ----snp_circos, fig.height=6-------------------------------------------------
plot_snp_circos(chr_length, gene_ranges, snp_pos)

## ----multiple_traits----------------------------------------------------------
# Simulate multiple traits by sampling 20 SNPs 4 times
snp_list <- GenomicRanges::GRangesList(
  Trait1 = sample(snp_pos, 20),
  Trait2 = sample(snp_pos, 20),
  Trait3 = sample(snp_pos, 20),
  Trait4 = sample(snp_pos, 20)
)

# Visualize SNP distribution across chromosomes
plot_snp_distribution(snp_list)

## ----snp_circos_multiple, fig.height=6----------------------------------------
# Visualize SNP positions in the genome as a circos plot
plot_snp_circos(chr_length, gene_ranges, snp_list)

## ----step_by_step-------------------------------------------------------------
candidates1 <- mine_step1(gene_ranges, snp_pos)
candidates1
length(candidates1)

## ----simulate_windows---------------------------------------------------------
# Single trait
simulate_windows(gene_ranges, snp_pos)

# Multiple traits
simulate_windows(gene_ranges, snp_list)

## ----step2--------------------------------------------------------------------
# Load guide genes
data(guides)
head(guides)

# Infer GCN
sft <- BioNERO::SFT_fit(pepper_se, net_type = "signed", cor_method = "pearson")
gcn <- BioNERO::exp2gcn(pepper_se, net_type = "signed", cor_method = "pearson",
                        module_merging_threshold = 0.8, SFTpower = sft$power)

# Apply step 2
candidates2 <- mine_step2(pepper_se, gcn = gcn, guides = guides$Gene,
                          candidates = candidates1$ID)
candidates2$candidates
candidates2$enrichment

## ----step3--------------------------------------------------------------------
# See the levels from the sample metadata
unique(pepper_se$Condition)

# Apply step 3 using "PRR_stress" as the condition of interest
candidates3 <- mine_step3(pepper_se, candidates = candidates2$candidates,
                          sample_group = "PRR_stress")
candidates3

## ----mine_candidates----------------------------------------------------------
candidates <- mine_candidates(gene_ranges = gene_ranges, 
                              marker_ranges = snp_pos, 
                              exp = pepper_se,
                              gcn = gcn, guides = guides$Gene,
                              sample_group = "PRR_stress")
candidates

## ----score_candidates---------------------------------------------------------
# Load TFs
data(tfs)
head(tfs)

# Get GCN hubs
hubs <- BioNERO::get_hubs_gcn(pepper_se, gcn)
head(hubs)

# Score candidates
scored <- score_genes(candidates, hubs$Gene, tfs$Gene_ID)
scored

## ----session_info-------------------------------------------------------------
sessioninfo::session_info()

#' Handle markers represented by intervals instead of SNPs
#'
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#' @param window Sliding window (in Mb) upstream and downstream relative
#' to each SNP. Default: 2.
#' @param expand_intervals Logical indicating whether or not to expand markers
#' that are represented by intervals. This is particularly useful
#' if users want to use a custom interval defined by linkage disequilibrium,
#' for example. Default: TRUE.
#'
#' @return A GRanges object with genomic intervals relative to each SNP.
#' @importFrom GenomicRanges width
#' @noRd
handle_intervals <- function(marker_ranges, window = 2,
                             expand_intervals = TRUE) {
    window <- window * 10^6
    widths <- GenomicRanges::width(marker_ranges)
    if(any(widths > 1) & expand_intervals == FALSE) {
        indices <- which(widths == 1)
        marker_window <- marker_ranges
        marker_window[indices] <- marker_window[indices] + window
    } else {
        marker_window <- marker_ranges + window
    }
    return(marker_window)
}

#' Step 1: Get all putative candidate genes for a given sliding window
#'
#' For a user-defined sliding window relative to each SNP, this function will
#' subset all genes whose genomic positions overlap with the sliding window.
#'
#' @param gene_ranges A GRanges object with genomic coordinates
#' of all genes in the genome.
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#' @param window Sliding window (in Mb) upstream and downstream relative
#' to each SNP. Default: 2.
#' @param expand_intervals Logical indicating whether or not to expand markers
#' that are represented by intervals. This is particularly useful
#' if users want to use a custom interval defined by linkage disequilibrium,
#' for example. Default: TRUE.
#'
#' @return A GRanges or GRangesList object with the genomic positions of
#' all putative candidate genes.
#' @seealso
#'  \code{\link[IRanges]{findOverlaps-methods}}
#' @rdname mine_step1
#' @export
#' @importFrom IRanges subsetByOverlaps
#' @importFrom GenomicRanges GRangesList
#' @examples
#' data(snp_pos)
#' data(gene_ranges)
#' genes <- mine_step1(gene_ranges, snp_pos, window = 2)
mine_step1 <- function(gene_ranges, marker_ranges, window = 2,
                       expand_intervals = TRUE) {
    allowed_classes <- c("GRanges", "GRangesList", "CompressedGRangesList")
    if(!class(marker_ranges) %in% allowed_classes) {
        stop("Argument 'marker_ranges' must be a GRanges, GRangesList, or
             CompressedGRangesList object.")
    }
    if(is(marker_ranges, "GRanges")) {
        snp_granges <- handle_intervals(marker_ranges, window = window,
                                        expand_intervals = expand_intervals)
        genes <- unique(IRanges::subsetByOverlaps(gene_ranges, snp_granges))
    } else {
        snp_granges <- lapply(marker_ranges, function(x) {
          return(handle_intervals(x, window = window,
                                  expand_intervals = expand_intervals))
        })
        genes <- lapply(snp_granges, function(x) {
            return(unique(IRanges::subsetByOverlaps(gene_ranges, x)))
        })
        genes <- GenomicRanges::GRangesList(genes)
    }
    return(genes)
}

#' Step 2: Get candidates in modules enriched in guide genes
#'
#' @param exp Expression data frame with genes in row names and samples in
#' column names or a SummarizedExperiment object.
#' @param gcn Gene coexpression network returned by \code{BioNERO::exp2gcn()}.
#' @param guides Guide genes as a character vector or as a data frame with
#' genes in the first column and gene annotation class in the second column.
#' @param candidates Character vector of all candidates genes to be inspected.
#' @param ... Additional arguments to \code{BioNERO::module_enrichment}
#'
#' @return A list of 2 elements:
#' \describe{
#'   \item{candidates}{Character vector of candidates after step 2}
#'   \item{enrichment}{Data frame of results for enrichment analysis}
#' }
#' @rdname mine_step2
#' @export
#' @importFrom BioNERO module_enrichment
#' @examples
#' data(pepper_se)
#' data(guides)
#' data(gcn)
#' set.seed(1)
#' mine2 <- mine_step2(
#'     exp = pepper_se,
#'     gcn = gcn,
#'     guides = guides$Gene,
#'     candidates = rownames(pepper_se)
#' )
mine_step2 <- function(exp, gcn, guides, candidates, ...) {
    if(is.character(guides)) {
        guides <- data.frame(Gene = guides, Class = "guide")
    }
    bkgenes <- rownames(exp)
    annotation <- merge(guides, as.data.frame(bkgenes), by = 1, all.y=TRUE)
    annotation[,2][is.na(annotation[,2])] <- "None"
    enrichment <- BioNERO::module_enrichment(
        gcn,
        background_genes = bkgenes,
        annotation = annotation,
        max_setsize = Inf,
        ...
    )
    enrichment <- enrichment[enrichment$term != "None", ]
    key_modules <- unique(enrichment$module)
    if(is.null(key_modules)) { stop("No modules enriched in guide genes.") }
    genes_f1 <- gcn$genes_and_modules$Genes[gcn$genes_and_modules$Modules %in%
                                                key_modules]
    genes_f1 <- genes_f1[genes_f1 %in% candidates]

    result_list <- list(
        candidates = genes_f1,
        enrichment = enrichment
    )
    return(result_list)
}

#' Step 3: Select candidates based on gene significance
#'
#' @param exp Expression data frame with genes in row names and samples in
#' column names or a SummarizedExperiment object.
#' @param metadata Sample metadata with samples in row names and sample
#' information in the first column. Ignored if `exp` is a SummarizedExperiment
#' object, as the colData will be extracted from the object.
#' @param metadata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column indices
#' (numeric) or column names (character). By default, all columns are used.
#' @param candidates Character vector of candidate genes to be inspected.
#' @param sample_group Level of sample metadata to be used for filtering
#' in gene-trait correlation.
#' @param min_cor Minimum correlation value for
#' \code{BioNERO::gene_significance()}. Default: 0.2
#' @param alpha Numeric indicating significance level. Default: 0.05
#' @param ... Additional arguments to \code{BioNERO::gene_significance}.
#'
#' @return A data frame with mined candidate genes and their correlation to
#' the condition of interest.
#'
#' @importFrom BioNERO gene_significance
#' @rdname mine_step3
#' @export
#' @examples
#' data(pepper_se)
#' data(snp_pos)
#' data(gene_ranges)
#' data(guides)
#' data(gcn)
#' data(mine2)
#' set.seed(1)
#' mine3 <- mine_step3(
#'     exp = pepper_se,
#'     candidates = mine2$candidates,
#'     sample_group = "PRR_stress"
#' )
mine_step3 <- function(
        exp, metadata, metadata_cols = 1,
        candidates, sample_group,
        min_cor = 0.2, alpha = 0.05,
        ...
) {
    exp <- exp[candidates, , drop = FALSE]
    genes_f2 <- BioNERO::gene_significance(
        exp,
        metadata = metadata,
        metadata_cols = metadata_cols,
        genes = candidates,
        alpha = alpha,
        min_cor = min_cor,
        ...
    )
    genes_f2 <- genes_f2[genes_f2$trait %in% sample_group, ]
    genes_final <- genes_f2[order(-genes_f2$cor), ]

    return(genes_final)
}

#' Mine high-confidence candidate genes in a single step
#'
#' @param gene_ranges A GRanges object with genomic coordinates
#' of all genes in the genome.
#' @param marker_ranges Genomic positions of SNPs. For a single trait,
#' a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
#' object, with each element of the list representing SNP positions for a
#' particular trait.
#' @param window Sliding window (in Mb) upstream and downstream relative
#' to each SNP. Default: 2.
#' @param expand_intervals Logical indicating whether or not to expand markers
#' that are represented by intervals. This is particularly useful
#' if users want to use a custom interval defined by linkage disequilibrium,
#' for example. Default: TRUE.
#' @param gene_col Column of the GRanges object containing gene ID.
#' Default: "ID", the default for gff/gff3 files imported with
#' rtracklayer::import.
#' @param exp Expression data frame with genes in row names and samples in
#' column names or a SummarizedExperiment object.
#' @param gcn Gene coexpression network returned by \code{BioNERO::exp2gcn()}.
#' @param guides Guide genes as a character vector or as a data frame with
#' genes in the first column and gene annotation class in the second column.
#' @param metadata Sample metadata with samples in row names and sample
#' information in the first column. Ignored if `exp` is a SummarizedExperiment
#' object, as the colData will be extracted from the object.
#' @param metadata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column indices
#' (numeric) or column names (character). By default, all columns are used.
#' @param sample_group Level of sample metadata to be used for filtering
#' in gene-trait correlation.
#' @param min_cor Minimum correlation value for
#' \code{BioNERO::gene_significance()}. Default: 0.2
#' @param alpha Numeric indicating significance level. Default: 0.05
#' @param ... Additional arguments to \code{BioNERO::gene_significance}.
#'
#' @return A data frame with mined candidate genes and their correlation to
#' the condition of interest.
#' @importFrom GenomicRanges mcols
#' @export
#' @rdname mine_candidates
#' @examples
#' \donttest{
#' data(pepper_se)
#' data(snp_pos)
#' data(gene_ranges)
#' data(guides)
#' data(gcn)
#' set.seed(1)
#' candidates <- mine_candidates(gene_ranges, snp_pos, exp = pepper_se,
#'                               gcn = gcn, guides = guides$Gene,
#'                               sample_group = "PRR_stress")
#' }
mine_candidates <- function(
        gene_ranges = NULL, marker_ranges = NULL,
        window = 2, expand_intervals = TRUE, gene_col = "ID",
        exp = NULL, gcn = NULL, guides = NULL,
        metadata, metadata_cols = 1, sample_group,
        min_cor = 0.2, alpha = 0.05,
        ...
) {

    # Step 1
    candidates1 <- mine_step1(
        gene_ranges = gene_ranges,
        marker_ranges = marker_ranges,
        window = window,
        expand_intervals = expand_intervals
    )
    candidates1 <- GenomicRanges::mcols(candidates1)[[gene_col]]

    # Step 2
    candidates2 <- mine_step2(
        exp = exp,
        gcn = gcn,
        guides = guides,
        candidates = candidates1
    )

    # Step 3
    candidates3 <- mine_step3(
        exp = exp,
        metadata = metadata,
        metadata_cols = metadata_cols,
        candidates = candidates2$candidates,
        sample_group = sample_group,
        min_cor = min_cor,
        alpha = alpha,
        ...
    )

    return(candidates3)
}


#' Score candidate genes and select the top n genes
#'
#' @param mined_candidates Data frame resulting from \code{mine_candidates()}
#' or \code{mine_step()}.
#' @param hubs Character vector of hub genes.
#' @param tfs Character vector of transcription factors.
#' @param pick_top Number of top genes to select. Default: 10.
#' @param weight_tf Numeric scalar with the weight to which correlation
#' coefficients will be multiplied if the gene is a TF. Default: 2.
#' @param weight_hub Numeric scalar with the weight to which correlation
#' coefficients will be multiplied if the gene is a hub. Default: 2.
#' @param weight_both Numeric scalar with the weight to which correlation
#' coefficients will be multiplied if the gene is both a TF and a hub.
#' Default: 3.
#'
#' @return Data frame with top n candidates and their scores.
#' @export
#' @rdname score_genes
#' @examples
#' data(tfs)
#' data(hubs)
#' data(mined_candidates)
#' set.seed(1)
#' scored <- score_genes(mined_candidates, hubs$Gene, tfs$Gene_ID)
score_genes <- function(mined_candidates, hubs = NULL, tfs = NULL,
                        pick_top=10,
                        weight_tf = 2, weight_hub = 2,
                        weight_both = 3) {
    if(is.null(hubs) & is.null(tfs)) {stop("Neither hubs nor TFs were provided.")}
    candidates <- mined_candidates
    cand_hubs <- candidates[candidates$gene %in% hubs, "gene"]
    cand_tfs <- candidates[candidates$gene %in% tfs, "gene"]
    cand_both <- intersect(cand_hubs, cand_tfs)
    scored <- candidates
    scored$score <- vapply(seq_len(nrow(scored)), function(x) {
        if(scored$gene[x] %in% cand_both) {
            y <- scored$cor[x] * weight_both
        } else if(scored$gene[x] %in% cand_hubs) {
            y <- scored$cor[x] * weight_hub
        } else if(scored$gene[x] %in% cand_tfs) {
            y <- scored$cor[x] * weight_tf
        } else {
            y <- scored$cor[x]
        }
        return(y)
    }, numeric(1))
    scored$abscore <- abs(scored$score)
    scored <- scored[order(-scored$abscore), ]
    scored <- scored[!duplicated(scored$gene), ]
    scored$abscore <- NULL
    if(nrow(scored) < pick_top) {
        message("Number of genes < 'pick_top'. Picking all genes.")
    } else {
        scored <- scored[1:pick_top, ]
    }
    return(scored)
}

#----Load data------------------------------------------------------------------
data(pepper_se)
data(snp_pos)
data(gene_ranges)
data(mined_candidates)
data(guides)
data(mine2)
data(tfs)
data(gcn)
data(hubs)

# Get candidates
genes <- mine_step1(gene_ranges, snp_pos, window = 2)$ID

# Create expression with only candidates
set.seed(1)

#----Start tests----------------------------------------------------------------
test_that("handle_intervals() returns a GRanges object", {
    sim_markers <- snp_pos + 50
    h1 <- handle_intervals(snp_pos)
    h2 <- handle_intervals(sim_markers, expand_intervals = FALSE)

    expect_true("GRanges" %in% class(h1))
    expect_true("GRanges" %in% class(h2))
})

test_that("mine_step1() returns a character vector of gene IDs", {
    genes <- mine_step1(gene_ranges, snp_pos, window = 2)
    snp_list <- GenomicRanges::GRangesList(
        Trait1 = sample(snp_pos, 20),
        Trait2 = sample(snp_pos, 20)
    )
    genes2 <- mine_step1(gene_ranges, snp_list)

    expect_true(is(genes, "GRanges"))
    expect_true(
        "GRangesList" %in% class(genes2) |
            "CompressedGRangesList" %in% class(genes2)
    )
    expect_error(mine_step1(gene_ranges, as.data.frame(snp_pos)))

})

test_that("mine_step2() returns a list with 2 elements", {
    mine2 <- mine_step2(
        pepper_se, gcn = gcn, guides = guides$Gene,
        candidates = rownames(pepper_se)
    )

    expect_equal(class(mine2), "list")
    expect_equal(names(mine2), c("candidates", "enrichment"))

    expect_error(
        mine_step2(
            pepper_se, gcn = gcn, guides = guides$Gene[1:5],
            candidates = rownames(pepper_se)
        )
    )
})

test_that("mine_step3() returns a data frame of mined candidates", {
    mine3 <- mine_step3(
        pepper_se, candidates = mine2$candidates,
        sample_group = "PRR_stress"
    )

    expect_equal(class(mine3), "data.frame")
})

test_that("mine_candidates() returns a data frame", {
    mc1 <- mine_candidates(
        gene_ranges, snp_pos, exp = pepper_se, gcn = gcn,
        guides = guides$Gene, sample_group = "PRR_stress"
    )

    expect_equal(class(mc1), "data.frame")
    expect_equal(ncol(mc1), 5)
})

test_that("score_genes() returns a data frame", {
    hc_genes <- mined_candidates
    scored <- score_genes(hc_genes, hubs$Gene, tfs$Gene_ID)

    # Artitficially add scored genes as hubs and TFs to test weighing
    scored2 <- score_genes(
        hc_genes,
        c(hubs$Gene, "CA12g18400", "CA03g03310"),
        c(tfs$Gene_ID, "CA03g03310", "CA10g02780")
    )

    scored3 <- score_genes(
        hc_genes,
        c(hubs$Gene, "CA12g18400", "CA03g03310"),
        c(tfs$Gene_ID, "CA03g03310", "CA10g02780"),
        pick_top = 2
    )

    expect_equal(class(scored), "data.frame")
    expect_equal(class(scored2), "data.frame")
    expect_equal(nrow(scored3), 2)
    expect_error(
        score_genes(hc_genes)
    )
})

---
title: "Mining high-confidence candidate genes with cageminer"
author: 
  - name: Fabricio Almeida-Silva
    affiliation: 
    - "Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil"
    - "Current address: VIB-UGent Center for Plant Systems Biology, Ghent University, Ghent, Belgium"
  - name: Thiago Motta Venancio
    affiliation: 
    - "Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil"
output: 
  BiocStyle::html_document:
    toc: true
    number_sections: yes
bibliography: vignette_bibliography.bib
vignette: >
  %\VignetteIndexEntry{Mining high-confidence candidate genes with cageminer}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL
)
```

# Introduction

Over the past years, RNA-seq data for several species have accumulated in public repositories. Additionally, genome-wide association studies (GWAS) have 
identified SNPs associated with phenotypes of interest, such as agronomic
traits in plants, production traits in livestock, and complex human diseases.
However, although GWAS can identify SNPs, they cannot identify causative genes
associated with the studied phenotype. The goal of `cageminer` is to integrate 
GWAS-derived SNPs with transcriptomic data to mine candidate genes and identify
high-confidence genes associated with traits of interest.

# Citation

If you use `cageminer` in your research, please cite us. You can obtain
citation information with `citation('cageminer')`, as demonstrated below:

```{r cit, eval = requireNamespace('cageminer')}
print(citation('cageminer'), bibtex = TRUE)
```

# Installation

```{r installation, eval=FALSE}
if(!requireNamespace('BiocManager', quietly = TRUE))
  install.packages('BiocManager')
BiocManager::install("cageminer")
```

```{r load_package, message=FALSE}
# Load package after installation
library(cageminer)
set.seed(123) # for reproducibility
```

# Data description

For this vignette, we will use transcriptomic data on pepper (*Capsicum annuum*)
response to Phytophthora root rot [@Kim2018], and GWAS SNPs associated with 
resistance to Phytophthora root rot from @Siddique2019. To ensure 
interoperability with other Bioconductor packages, expression data are 
stored as SummarizedExperiment objects, and gene/SNP positions are stored as
GRanges objects.

```{r data_description}
# GRanges of SNP positions
data(snp_pos)
snp_pos

# GRanges of chromosome lengths
data(chr_length) 
chr_length

# GRanges of gene coordinates
data(gene_ranges) 
gene_ranges

# SummarizedExperiment of pepper response to Phytophthora root rot (RNA-seq)
data(pepper_se)
pepper_se
```

# Visualizing SNP distribution

Before mining high-confidence candidates, you can visualize the SNP distribution
in the genome to explore possible patterns. First, let's see if SNPs are
uniformly across chromosomes with `plot_snp_distribution()`.

```{r snp_dist}
plot_snp_distribution(snp_pos)
```

As we can see, SNPs associated with resistance to Phytophthora root rot tend to
co-occur in chromosome 5. Now, we can see if they are close to each other in the
genome, and if they are located in gene-rich regions. We can visualize it with
`plot_snp_circos`, which displays a circos plot of SNPs across chromosomes.

```{r snp_circos, fig.height=6}
plot_snp_circos(chr_length, gene_ranges, snp_pos)
```

There seems to be no clustering in gene-rich regions, but we can see that SNPs 
in the same chromosome tend to be physically close to each other.

If you have SNP positions for multiple traits, you need to store them in
GRangesList or CompressedGRangesList objects, so each element will have SNP
positions for a particular trait. Then, you can visualize their distribution
as you would do for a single trait. Let's simulate multiple traits to see
how it works:

```{r multiple_traits}
# Simulate multiple traits by sampling 20 SNPs 4 times
snp_list <- GenomicRanges::GRangesList(
  Trait1 = sample(snp_pos, 20),
  Trait2 = sample(snp_pos, 20),
  Trait3 = sample(snp_pos, 20),
  Trait4 = sample(snp_pos, 20)
)

# Visualize SNP distribution across chromosomes
plot_snp_distribution(snp_list)
```

```{r snp_circos_multiple, fig.height=6}
# Visualize SNP positions in the genome as a circos plot
plot_snp_circos(chr_length, gene_ranges, snp_list)
```

# Algorithm description

The `cageminer` algorithm identifies high-confidence candidate genes with
**3 steps**, which can be interpreted as 3 sources of evidence:

1. Select all genes in a sliding window relative to each SNP as putative
candidates.
2. Find candidates from step 1 in coexpression modules enriched in guide genes 
(genes that are known to be associated with the trait of interest).
3. Find candidates from step 2 that are correlated with a condition of interest.

These 3 steps can be executed individually (if users want more control on 
what happens after each step) or all at once.

# Step-by-step candidate gene mining

To run the candidate mining step by step, you will need the functions 
`mine_step1()`, `mine_step2`, and `mine_step3`.

## Step 1: finding genes close to (or in linkage disequilibrium with) SNPs

The function `mine_step1()` identifies genes based on step 1 and returns a
GRanges object with all putative candidates and their location in the genome.
For that, you need to give 2 GRanges objects as input, one with the 
gene coordinates[^1] and another with the SNP positions.

[^1]: **Tip:** to create GRanges objects from genomic coordinates in GFF/GTF 
files, you can use the `import()` function from the Bioconductor package 
rtracklayer [@rtracklayer2009].

```{r step_by_step}
candidates1 <- mine_step1(gene_ranges, snp_pos)
candidates1
length(candidates1)
```

The first step identified `r length(candidates1)` putative candidate genes. 
By default, `cageminer` uses a sliding window of 2 Mb to select putative 
candidates[^2]. If you want to visually inspect a simulation of different 
sliding windows to choose a different one, you can use `simulate_windows()`.

```{r simulate_windows}
# Single trait
simulate_windows(gene_ranges, snp_pos)

# Multiple traits
simulate_windows(gene_ranges, snp_list)
```

[^2]: **Note:** By default, SNPs coordinates will be expanded upstream and 
downstream according to the input window size. However, you may have 
previously determined genomic intervals for each SNP (e.g., calculated based on
linkage disequilibrium) for which you want to extract genes. To avoid expanding
a sliding window in such cases, set `expand_intervals = FALSE`. This will
ensure that only SNPs are expanded, but not intervals (width >1).

## Step 2: finding coexpression modules enriched in guide genes

The function `mine_step2()` selects candidates in coexpression modules enriched 
in guide genes. For that, users must infer the GCN with the function exp2gcn() 
from the package BioNERO [@R-BioNERO]. Guide genes can be either a character 
vector of guide gene IDs or a data frame with gene IDs in the first column 
and annotation in the second column (useful if guides are divided in functional
categories, for instance). Here, pepper genes associated with defense-related
MapMan bins were retrieved from PLAZA 3.0 Dicots [@Proost2015] and used as
guides.

The resulting object is a list of two elements: 

- *candidates:* character vector of mined candidate gene IDs.
- *enrichment:* data frame of enrichment results.

```{r step2}
# Load guide genes
data(guides)
head(guides)

# Infer GCN
sft <- BioNERO::SFT_fit(pepper_se, net_type = "signed", cor_method = "pearson")
gcn <- BioNERO::exp2gcn(pepper_se, net_type = "signed", cor_method = "pearson",
                        module_merging_threshold = 0.8, SFTpower = sft$power)

# Apply step 2
candidates2 <- mine_step2(pepper_se, gcn = gcn, guides = guides$Gene,
                          candidates = candidates1$ID)
candidates2$candidates
candidates2$enrichment
```

After the step 2, we got `r length(candidates2$candidates)` candidates.

## Step 3: finding genes with altered expression in a condition of interest

The function `mine_step3()` identifies candidate genes whose expression levels
significantly increase or decrease in a particular condition. For that, you 
need to specify what level from the sample metadata corresponds to this 
condition. The resulting object from `mine_step3()` is a data frame with mined
candidates and their correlation to the condition of interest.

```{r step3}
# See the levels from the sample metadata
unique(pepper_se$Condition)

# Apply step 3 using "PRR_stress" as the condition of interest
candidates3 <- mine_step3(pepper_se, candidates = candidates2$candidates,
                          sample_group = "PRR_stress")
candidates3
```

Finally, we got `r length(unique(candidates3$gene))` high-confidence 
candidate genes associated with resistance to Phytophthora root rot. Genes with
negative correlation coefficients to the condition can be interpreted as
having significantly reduced expression in this condition, while genes with 
positive correlation coefficients have significantly increased expression in
this condition.

# Automatic candidate gene mining

Alternatively, if you are not interested in inspecting the results after each
step, you can get to the same results from the previous section with a single
step by using the function `mine_candidates()`. This function is a wrapper that
calls `mine_step1()`, sends the results to `mine_step2()`, and then it sends
the results from `mine_step2()` to `mine_step3()`.

```{r mine_candidates}
candidates <- mine_candidates(gene_ranges = gene_ranges, 
                              marker_ranges = snp_pos, 
                              exp = pepper_se,
                              gcn = gcn, guides = guides$Gene,
                              sample_group = "PRR_stress")
candidates
```

# Score candidates

In some cases, you might have more high-confidence candidates than you expected,
and you want to pick only the top *n* genes for validation, for instance. In
this scenario, you need to assign scores to your mined candidates to pick the
top *n* genes with the highest scores. The function `score_genes()` does
that by using the formula below:

$$S_i = r_{pb} \kappa$$

where:

$\kappa$ = 2 if the gene encodes a transcription factor

$\kappa$ = 2 if the gene is a hub

$\kappa$ = 3 if the gene encodes a hub transcription factor

$\kappa$ = 1 if none of the conditions above are true


By default, `score_genes` picks the top 10 candidates. If there are less than 10 candidates, it will return all candidates sorted by scores. Here, TFs were obtained from PlantTFDB 4.0 [@Jin2017]. Hub genes can be 
identified with the function `get_hubs_gcn()` from the package BioNERO.

```{r score_candidates}
# Load TFs
data(tfs)
head(tfs)

# Get GCN hubs
hubs <- BioNERO::get_hubs_gcn(pepper_se, gcn)
head(hubs)

# Score candidates
scored <- score_genes(candidates, hubs$Gene, tfs$Gene_ID)
scored
```

As none of the mined candidates are hubs or encode transcription factors,
their scores are simply their correlation coefficients with the condition of
interest.

# Session information {.unnumbered}

This document was created under the following conditions:

```{r session_info}
sessioninfo::session_info()
```

# References {.unnumbered}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gcn}
\alias{gcn}
\title{Simulation of the output list from BioNERO::exp2gcn() with pepper data}
\format{
A list with the elements returned by BioNERO::exp2gcn().
}
\usage{
data(gcn)
}
\description{
This object is a list as returned by BioNERO::exp2gcn(), but only the
element genes_and_modules is included. For running time issues,
only genes in the cyan module were kept in the element genes_and_modules.
All other list elements have been assigned NULL.
The network was inferred using the code from the vignette.
}
\examples{
data(gcn)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/candidate_mining.R
\name{score_genes}
\alias{score_genes}
\title{Score candidate genes and select the top n genes}
\usage{
score_genes(
  mined_candidates,
  hubs = NULL,
  tfs = NULL,
  pick_top = 10,
  weight_tf = 2,
  weight_hub = 2,
  weight_both = 3
)
}
\arguments{
\item{mined_candidates}{Data frame resulting from \code{mine_candidates()}
or \code{mine_step()}.}

\item{hubs}{Character vector of hub genes.}

\item{tfs}{Character vector of transcription factors.}

\item{pick_top}{Number of top genes to select. Default: 10.}

\item{weight_tf}{Numeric scalar with the weight to which correlation
coefficients will be multiplied if the gene is a TF. Default: 2.}

\item{weight_hub}{Numeric scalar with the weight to which correlation
coefficients will be multiplied if the gene is a hub. Default: 2.}

\item{weight_both}{Numeric scalar with the weight to which correlation
coefficients will be multiplied if the gene is both a TF and a hub.
Default: 3.}
}
\value{
Data frame with top n candidates and their scores.
}
\description{
Score candidate genes and select the top n genes
}
\examples{
data(tfs)
data(hubs)
data(mined_candidates)
set.seed(1)
scored <- score_genes(mined_candidates, hubs$Gene, tfs$Gene_ID)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mined_candidates}
\alias{mined_candidates}
\title{Example output from mined_candidates()}
\format{
Data frame with an example of the output from mined_candidates
}
\usage{
data(mined_candidates)
}
\description{
The data frame was created using the code from the vignette.
}
\examples{
data(mined_candidates)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gene_ranges}
\alias{gene_ranges}
\title{Genomic coordinates of pepper genes}
\format{
A GRanges object
}
\usage{
data(gene_ranges)
}
\description{
GRanges object with genomic coordinates of pepper genes downloaded from
http://peppergenome.snu.ac.kr/download.php.
}
\examples{
data(gene_ranges)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/candidate_mining.R
\name{mine_step1}
\alias{mine_step1}
\title{Step 1: Get all putative candidate genes for a given sliding window}
\usage{
mine_step1(gene_ranges, marker_ranges, window = 2, expand_intervals = TRUE)
}
\arguments{
\item{gene_ranges}{A GRanges object with genomic coordinates
of all genes in the genome.}

\item{marker_ranges}{Genomic positions of SNPs. For a single trait,
a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
object, with each element of the list representing SNP positions for a
particular trait.}

\item{window}{Sliding window (in Mb) upstream and downstream relative
to each SNP. Default: 2.}

\item{expand_intervals}{Logical indicating whether or not to expand markers
that are represented by intervals. This is particularly useful
if users want to use a custom interval defined by linkage disequilibrium,
for example. Default: TRUE.}
}
\value{
A GRanges or GRangesList object with the genomic positions of
all putative candidate genes.
}
\description{
For a user-defined sliding window relative to each SNP, this function will
subset all genes whose genomic positions overlap with the sliding window.
}
\examples{
data(snp_pos)
data(gene_ranges)
genes <- mine_step1(gene_ranges, snp_pos, window = 2)
}
\seealso{
\code{\link[IRanges]{findOverlaps-methods}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{chr_length}
\alias{chr_length}
\title{Pepper chromosome lengths}
\format{
A GRanges object
}
\usage{
data(chr_length)
}
\description{
Lengths of pepper chromosomes 1-12 in a GRanges object.
The genome for which lengths were calculated (v1.55) was downloaded from
http://peppergenome.snu.ac.kr/download.php
}
\examples{
data(chr_length)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomic_intervals_operations.R
\name{simulate_windows}
\alias{simulate_windows}
\title{Simulate number of genes for each sliding window}
\usage{
simulate_windows(
  gene_ranges,
  marker_ranges,
  windows = seq(0.1, 2, by = 0.1),
  expand_intervals = TRUE
)
}
\arguments{
\item{gene_ranges}{A GRanges object with genomic coordinates
of all genes in the genome.}

\item{marker_ranges}{Genomic positions of SNPs. For a single trait,
a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
object, with each element of the list representing SNP positions for a
particular trait.}

\item{windows}{Sliding windows (in Mb) upstream and downstream relative
to each SNP. Default: seq(0.1, 2, by = 0.1).}

\item{expand_intervals}{Logical indicating whether or not to expand markers
that are represented by intervals. This is particularly useful
if users want to use a custom interval defined by linkage disequilibrium,
for example. Default: TRUE.}
}
\value{
A ggplot object summarizing the results of the simulations.
}
\description{
This function counts genes that are contained in sliding windows related to
each SNP.
}
\details{
By default, the function creates 20 sliding windows by expanding upstream
and downstream boundaries for each SNP from 0.1 Mb (100 kb) to 2 Mb.
}
\examples{
data(snp_pos)
data(gene_ranges)
simulate_windows(gene_ranges, snp_pos)
}
\seealso{
\code{\link[IRanges]{findOverlaps-methods}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/candidate_mining.R
\name{mine_step3}
\alias{mine_step3}
\title{Step 3: Select candidates based on gene significance}
\usage{
mine_step3(
  exp,
  metadata,
  metadata_cols = 1,
  candidates,
  sample_group,
  min_cor = 0.2,
  alpha = 0.05,
  ...
)
}
\arguments{
\item{exp}{Expression data frame with genes in row names and samples in
column names or a SummarizedExperiment object.}

\item{metadata}{Sample metadata with samples in row names and sample
information in the first column. Ignored if \code{exp} is a SummarizedExperiment
object, as the colData will be extracted from the object.}

\item{metadata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a \code{SummarizedExperiment} object. The vector can contain column indices
(numeric) or column names (character). By default, all columns are used.}

\item{candidates}{Character vector of candidate genes to be inspected.}

\item{sample_group}{Level of sample metadata to be used for filtering
in gene-trait correlation.}

\item{min_cor}{Minimum correlation value for
\code{BioNERO::gene_significance()}. Default: 0.2}

\item{alpha}{Numeric indicating significance level. Default: 0.05}

\item{...}{Additional arguments to \code{BioNERO::gene_significance}.}
}
\value{
A data frame with mined candidate genes and their correlation to
the condition of interest.
}
\description{
Step 3: Select candidates based on gene significance
}
\examples{
data(pepper_se)
data(snp_pos)
data(gene_ranges)
data(guides)
data(gcn)
data(mine2)
set.seed(1)
mine3 <- mine_step3(
    exp = pepper_se,
    candidates = mine2$candidates,
    sample_group = "PRR_stress"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pepper_se}
\alias{pepper_se}
\title{Gene expression data from Kim et al., 2018.}
\format{
A SummarizedExperiment object.
}
\usage{
data(pepper_se)
}
\description{
The data were filtered to keep only the top 4000 genes with highest RPKM
values in PRR stress-related samples.
}
\examples{
data(pepper_se)
}
\references{
Kim, MS., Kim, S., Jeon, J. et al. Global gene expression profiling for
fruit organs and pathogen infections in the pepper, Capsicum annuum L..
Sci Data 5, 180103 (2018). https://doi.org/10.1038/sdata.2018.103
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tfs}
\alias{tfs}
\title{Pepper transcription factors}
\format{
A data frame with gene IDs in the first column and TF families in
the second column.
}
\usage{
data(tfs)
}
\description{
Pepper transcription factors and their families retrieved from PlantTFDB 4.0.
}
\examples{
data(tfs)
}
\references{
Jin, J., Tian, F., Yang, D. C., Meng, Y. Q., Kong, L., Luo, J., &
Gao, G. (2016). PlantTFDB 4.0: toward a central hub for transcription
factors and regulatory interactions in plants. Nucleic acids research, gkw982.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hubs}
\alias{hubs}
\title{Example hub genes for the network stored in the gcn object}
\format{
Data frame with gene IDs, module and intramodular degree.
}
\usage{
data(hubs)
}
\description{
The data frame was created using the code from the vignette.
}
\examples{
data(hubs)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{snp_pos}
\alias{snp_pos}
\title{Capsicum annuum SNPs associated with resistance to Phytophthora root rot.}
\format{
A GRanges object.
}
\usage{
data(snp_pos)
}
\description{
The SNPs in this data set were retrieved from Siddique et al., 2019, and
they are associated to resistance to Phytophthora root rot.
}
\examples{
data(snp_pos)
}
\references{
Siddique, M.I., Lee, HY., Ro, NY. et al. Identifying candidate genes for
Phytophthora capsici resistance in pepper (Capsicum annuum) via
genotyping-by-sequencing-based QTL mapping and genome-wide association
study. Sci Rep 9, 9962 (2019). https://doi.org/10.1038/s41598-019-46342-1
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_snp_distribution}
\alias{plot_snp_distribution}
\title{Plot a barplot of SNP distribution across chromosomes}
\usage{
plot_snp_distribution(marker_ranges)
}
\arguments{
\item{marker_ranges}{Genomic positions of SNPs. For a single trait,
a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
object, with each element of the list representing SNP positions for a
particular trait. List elements must have names for proper labelling.}
}
\value{
A ggplot object.
}
\description{
Plot a barplot of SNP distribution across chromosomes
}
\examples{
data(snp_pos)
p <- plot_snp_distribution(snp_pos)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_snp_circos}
\alias{plot_snp_circos}
\title{Circos plot of SNP distribution across chromosomes}
\usage{
plot_snp_circos(genome_ranges, gene_ranges, marker_ranges)
}
\arguments{
\item{genome_ranges}{A GRanges object with chromosome lengths.}

\item{gene_ranges}{A GRanges object with genomic coordinates
of all genes in the genome.}

\item{marker_ranges}{Genomic positions of SNPs. For a single trait,
a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
object, with each element of the list representing SNP positions for a
particular trait.}
}
\value{
A ggplot object with a circos plot of molecular marker distribution
across chromosomes.
}
\description{
Circos plot of SNP distribution across chromosomes
}
\examples{
data(snp_pos)
data(gene_ranges)
data(chr_length)
p <- plot_snp_circos(chr_length, gene_ranges, snp_pos)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mine2}
\alias{mine2}
\title{Example output from mine_step2()}
\format{
List with elements 'candidates' (character vector) and 'enrichment'
(data frame).
}
\usage{
data(mine2)
}
\description{
The list was created using the example code from mine_step().
}
\examples{
data(mine2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/candidate_mining.R
\name{mine_candidates}
\alias{mine_candidates}
\title{Mine high-confidence candidate genes in a single step}
\usage{
mine_candidates(
  gene_ranges = NULL,
  marker_ranges = NULL,
  window = 2,
  expand_intervals = TRUE,
  gene_col = "ID",
  exp = NULL,
  gcn = NULL,
  guides = NULL,
  metadata,
  metadata_cols = 1,
  sample_group,
  min_cor = 0.2,
  alpha = 0.05,
  ...
)
}
\arguments{
\item{gene_ranges}{A GRanges object with genomic coordinates
of all genes in the genome.}

\item{marker_ranges}{Genomic positions of SNPs. For a single trait,
a GRanges object. For multiple traits, a GRangesList or CompressedGRangesList
object, with each element of the list representing SNP positions for a
particular trait.}

\item{window}{Sliding window (in Mb) upstream and downstream relative
to each SNP. Default: 2.}

\item{expand_intervals}{Logical indicating whether or not to expand markers
that are represented by intervals. This is particularly useful
if users want to use a custom interval defined by linkage disequilibrium,
for example. Default: TRUE.}

\item{gene_col}{Column of the GRanges object containing gene ID.
Default: "ID", the default for gff/gff3 files imported with
rtracklayer::import.}

\item{exp}{Expression data frame with genes in row names and samples in
column names or a SummarizedExperiment object.}

\item{gcn}{Gene coexpression network returned by \code{BioNERO::exp2gcn()}.}

\item{guides}{Guide genes as a character vector or as a data frame with
genes in the first column and gene annotation class in the second column.}

\item{metadata}{Sample metadata with samples in row names and sample
information in the first column. Ignored if \code{exp} is a SummarizedExperiment
object, as the colData will be extracted from the object.}

\item{metadata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a \code{SummarizedExperiment} object. The vector can contain column indices
(numeric) or column names (character). By default, all columns are used.}

\item{sample_group}{Level of sample metadata to be used for filtering
in gene-trait correlation.}

\item{min_cor}{Minimum correlation value for
\code{BioNERO::gene_significance()}. Default: 0.2}

\item{alpha}{Numeric indicating significance level. Default: 0.05}

\item{...}{Additional arguments to \code{BioNERO::gene_significance}.}
}
\value{
A data frame with mined candidate genes and their correlation to
the condition of interest.
}
\description{
Mine high-confidence candidate genes in a single step
}
\examples{
\donttest{
data(pepper_se)
data(snp_pos)
data(gene_ranges)
data(guides)
data(gcn)
set.seed(1)
candidates <- mine_candidates(gene_ranges, snp_pos, exp = pepper_se,
                              gcn = gcn, guides = guides$Gene,
                              sample_group = "PRR_stress")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/candidate_mining.R
\name{mine_step2}
\alias{mine_step2}
\title{Step 2: Get candidates in modules enriched in guide genes}
\usage{
mine_step2(exp, gcn, guides, candidates, ...)
}
\arguments{
\item{exp}{Expression data frame with genes in row names and samples in
column names or a SummarizedExperiment object.}

\item{gcn}{Gene coexpression network returned by \code{BioNERO::exp2gcn()}.}

\item{guides}{Guide genes as a character vector or as a data frame with
genes in the first column and gene annotation class in the second column.}

\item{candidates}{Character vector of all candidates genes to be inspected.}

\item{...}{Additional arguments to \code{BioNERO::module_enrichment}}
}
\value{
A list of 2 elements:
\describe{
\item{candidates}{Character vector of candidates after step 2}
\item{enrichment}{Data frame of results for enrichment analysis}
}
}
\description{
Step 2: Get candidates in modules enriched in guide genes
}
\examples{
data(pepper_se)
data(guides)
data(gcn)
set.seed(1)
mine2 <- mine_step2(
    exp = pepper_se,
    gcn = gcn,
    guides = guides$Gene,
    candidates = rownames(pepper_se)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{guides}
\alias{guides}
\title{Guide genes associated with defense and resistance to oomycetes}
\format{
A data frame with genes in the first column and GO description
in the second column.
}
\usage{
data(guides)
}
\description{
The GO annotation was retrieved from PLAZA 4.0 Dicots.
}
\examples{
data(guides)
}
\references{
Van Bel, M., Diels, T., Vancaester, E., Kreft, L., Botzki, A.,
Van de Peer, Y., ... & Vandepoele, K. (2018). PLAZA 4.0: an integrative
resource for functional, evolutionary and comparative plant
genomics. Nucleic acids research, 46(D1), D1190-D1196.
}
\keyword{datasets}
